import datetime as dt
from typing import Literal, Optional
from urllib.parse import urlencode

from pydantic import (
    BaseModel,
    Field,
    NonNegativeInt,
    PositiveInt,
)

from brreg.enhetsregisteret._types import (
    CommaList,
    Kommunenummer,
    Naeringskode,
    Organisasjonsnummer,
    Postnummer,
    Sektorkode,
)

__all__ = [
    "EnhetQuery",
    "UnderenhetQuery",
]


class Query(BaseModel):
    """The fields here are available on all queries."""

    #: Sortering av resultatsett
    sort: Optional[Literal["ASC", "DESC"]] = None

    #: Sidestørrelse
    size: Optional[PositiveInt] = None

    #: Sidenummer
    page: Optional[NonNegativeInt] = None

    def as_url_query(self) -> str:
        params = self.model_dump(
            mode="json",
            by_alias=True,
            exclude_defaults=True,
        )
        for key, value in params.items():
            if isinstance(value, bool):
                params[key] = str(value).lower()
        return urlencode(params)


class EnhetQuery(Query):
    """The query type for enhet search."""

    #: Enhetens navn
    navn: Optional[str] = None

    #: Organisasjonsnummeret til enheten
    organisasjonsnummer: CommaList[Organisasjonsnummer] = Field(
        default_factory=list,
    )

    #: Organisasjonsnummeret til enhetens overordnede enhet
    overordnet_enhet: Optional[Organisasjonsnummer] = Field(
        default=None,
        serialization_alias="overordnetEnhet",
    )

    #: Minste antall ansatte hos enheten
    fra_antall_ansatte: Optional[PositiveInt] = Field(
        default=None,
        serialization_alias="fraAntallAnsatte",
    )

    #: Største antall ansatte hos enheten
    til_antall_ansatte: Optional[PositiveInt] = Field(
        default=None,
        serialization_alias="tilAntallAnsatte",
    )

    #: Hvorvidt enheten er registrert som konkurs
    konkurs: Optional[bool] = None

    #: Hvorvidt enheten er registrert i Mva-registeret
    registrert_i_mvaregisteret: Optional[bool] = Field(
        default=None,
        serialization_alias="registrertIMvaregisteret",
    )

    #: Hvorvidt enheten er registrert i Foretaksregisteret
    registert_i_foretaksregisteret: Optional[bool] = Field(
        default=None,
        serialization_alias="registrertIForetaksregisteret",
    )

    #: Hvorvidt enheten er registrert i Stiftelsesregisteret
    registrert_i_stiftelsesregisteret: Optional[bool] = Field(
        default=None,
        serialization_alias="registrertIStiftelsesregisteret",
    )

    #: Hvorvidt enheten er registrert i Frivillighetsregisteret
    registrert_i_frivillighetsregisteret: Optional[bool] = Field(
        default=None,
        serialization_alias="registrertIFrivillighetsregisteret",
    )

    #: Frivillig registrert i Merverdiavgiftsregisteret
    frivillig_registrert_i_mvaregisteret: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="frivilligRegistrertIMvaregisteret",
    )

    #: Hvorvidt enheten er registrert som under tvangsavvikling eller
    #: tvangsopplosning
    under_tvangsavvikling_eller_trangsopplosning: Optional[bool] = None

    #: Hvorvidt enheten er registrert som under avvikling
    under_avvikling: Optional[bool] = None

    #: Tidligste registreringsdato i Enhetsregisteret
    fra_registreringsdato_enhetsregisteret: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraRegistreringsdatoEnhetsregisteret",
    )

    #: Seneste registreringsdato i Enhetsregisteret
    til_registreringsdato_enhetsregisteret: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilRegistreringsdatoEnhetsregisteret",
    )

    #: Tidligste stiftelsesdato for enheten
    fra_stiftelsesdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraStiftelsesdato",
    )

    #: Seneste stiftelsesdato for enheten
    til_stiftelsesdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilStiftelsesdato",
    )

    #: Enhetens organisasjonsform
    organisasjonsform: CommaList[str] = Field(
        default_factory=list,
    )

    #: Enhetens hjemmeside
    hjemmeside: Optional[str] = None

    #: Enhetens institusjonelle sektorkode
    institusjonell_sektorkode: CommaList[Sektorkode] = Field(
        default_factory=list,
    )

    #: Kommunenummer til enhetens postadresse
    postadresse_kommunenummer: CommaList[Kommunenummer] = Field(
        default_factory=list,
        serialization_alias="postadresse.kommunenummer",
    )

    #: Postnummeret til enhetens postadresse
    postadresse_postnummer: CommaList[Postnummer] = Field(
        default_factory=list,
        serialization_alias="postadresse.postnummer",
    )

    #: Poststedet til enhetens postadresse
    postadresse_poststed: Optional[str] = Field(
        default=None,
        serialization_alias="postadresse.poststed",
    )

    #: Landkode til enhetens postadresse
    postadresse_landkode: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="postadresse.landkode",
    )

    #: Adresse til enhetens postadresse
    postadresse_adresse: Optional[str] = Field(
        default=None,
        serialization_alias="postadresse.adresse",
    )

    #: Kommunenummer til enhetens forretningsadresse
    kommunenummer: CommaList[Kommunenummer] = Field(
        default_factory=list,
        serialization_alias="kommunenummer",
    )

    #: Postnummeret til enhetens forretningsadresse
    forretningsadresse_postnummer: CommaList[Postnummer] = Field(
        default_factory=list,
        serialization_alias="forretningsadresse.postnummer",
    )

    #: Poststedet til enhetens forretningsadresse
    forretningsadresse_poststed: Optional[str] = Field(
        default=None,
        serialization_alias="forretningsadresse.poststed",
    )

    #: Landkode til enhetens forretningsadresse
    forretningsadresse_landkode: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="forretningsadresse.landkode",
    )

    #: Adresse til enhetens forretningsadresse
    forretningsadresse_adresse: Optional[str] = Field(
        default=None,
        serialization_alias="forretningsadresse.adresse",
    )

    #: Enhetens næringskode
    naeringskode: CommaList[Naeringskode] = Field(
        default_factory=list,
    )

    #: Årstall for siste innsendte årsregnskap for enheten
    siste_innsendte_aarsregnskap: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="sisteInnsendteAarsregnskap",
    )


class UnderenhetQuery(Query):
    """The query type for underenhet search."""

    #: Underenhetens navn
    navn: Optional[str] = None

    #: Organisasjonsnummeret til underenheten
    organisasjonsnummer: CommaList[Organisasjonsnummer] = Field(
        default_factory=list,
    )

    #: Organisasjonsnummeret til underenhetens overordnede enhet
    overordnet_enhet: Optional[Organisasjonsnummer] = Field(
        default=None,
        serialization_alias="overordnetEnhet",
    )

    #: Minste antall ansatte hos underenheten
    fra_antall_ansatte: Optional[PositiveInt] = Field(
        default=None,
        serialization_alias="fraAntallAnsatte",
    )

    #: Største antall ansatte hos underenheten
    til_antall_ansatte: Optional[PositiveInt] = Field(
        default=None,
        serialization_alias="tilAntallAnsatte",
    )

    #: Hvorvidt underenheten er registrert i Mva-registeret
    registrert_i_mvaregisteret: Optional[bool] = Field(
        default=None,
        serialization_alias="registrertIMvaregisteret",
    )

    #: Tidligste registreringsdato i Enhetsregisteret
    fra_registreringsdato_enhetsregisteret: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraRegistreringsdatoEnhetsregisteret",
    )

    #: Seneste registreringsdato i Enhetsregisteret
    til_registreringsdato_enhetsregisteret: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilRegistreringsdatoEnhetsregisteret",
    )

    #: Tidligste oppstartsdato for enheten
    fra_oppstartsdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraOppstartsdato",
    )

    #: Seneste oppstartsdato for enheten
    til_oppstartsdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilOppstartsdato",
    )

    #: Tidligste registreringsdato for eierskifte
    fra_dato_eierskifte: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraDatoEierskifte",
    )

    #: Seneste registreringsdato for eierskifte
    til_dato_eierskifte: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilDatoEierskifte",
    )

    #: Tidligste nedleggelsesdato for enheten
    fra_nedleggelsesdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="fraNedleggelsesdato",
    )

    #: Seneste nedleggelsesdato for enheten
    til_nedleggelsesdato: Optional[dt.date] = Field(
        default=None,
        serialization_alias="tilNedleggelsesdato",
    )

    #: Underenhetens organisasjonsform
    organisasjonsform: CommaList[str] = Field(
        default_factory=list,
    )

    #: Enhetens hjemmeside
    hjemmeside: Optional[str] = None

    #: Kommunenummer til underenhetens postadresse
    postadresse_kommunenummer: CommaList[Kommunenummer] = Field(
        default_factory=list,
        serialization_alias="postadresse.kommunenummer",
    )

    #: Postnummeret til underenhetens postadresse
    postadresse_postnummer: CommaList[Postnummer] = Field(
        default_factory=list,
        serialization_alias="postadresse.postnummer",
    )

    #: Poststedet til underenhetens postadresse
    postadresse_poststed: Optional[str] = Field(
        default=None,
        serialization_alias="postadresse.poststed",
    )

    #: Landkode til underenhetens postadresse
    postadresse_landkode: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="postadresse.landkode",
    )

    #: Adresse til underenhetens postadresse
    postadresse_adresse: Optional[str] = Field(
        default=None,
        serialization_alias="postadresse.adresse",
    )

    #: Kommunenummer til enhetens beliggenhetsadresse
    kommunenummer: CommaList[Kommunenummer] = Field(
        default_factory=list,
        serialization_alias="kommunenummer",
    )

    #: Postnummeret til enhetens beliggenhetsadresse
    beliggenhetsadresse_postnummer: CommaList[Postnummer] = Field(
        default_factory=list,
        serialization_alias="beliggenhetsadresse.postnummer",
    )

    #: Poststedet til enhetens beliggenhetsadresse
    beliggenhetsadresse_poststed: Optional[str] = Field(
        default=None,
        serialization_alias="beliggenhetsadresse.poststed",
    )

    #: Landkode til enhetens beliggenhetsadresse
    beliggenhetsadresse_landkode: CommaList[str] = Field(
        default_factory=list,
        serialization_alias="beliggenhetsadresse.landkode",
    )

    #: Adresse til enhetens beliggenhetsadresse
    beliggenhetsadresse_adresse: Optional[str] = Field(
        default=None,
        serialization_alias="beliggenhetsadresse.adresse",
    )

    #: Underenhetens næringskode
    naeringskode: CommaList[Naeringskode] = Field(
        default_factory=list,
    )



from __future__ import annotations

from contextlib import contextmanager
from typing import TYPE_CHECKING, Any, Generator, Optional

import httpx

from brreg import BrregError, BrregRestError
from brreg.enhetsregisteret._pagination import EnhetPage, UnderenhetPage
from brreg.enhetsregisteret._responses import Enhet, Underenhet
from brreg.enhetsregisteret._types import (
    Organisasjonsnummer,
    OrganisasjonsnummerValidator,
)

if TYPE_CHECKING:
    from types import TracebackType

    from brreg.enhetsregisteret._queries import EnhetQuery, UnderenhetQuery


class Client:
    """Client for the Enhetregisteret API.

    Ensures that HTTP connections are reused across requests.

    It can be used as a context manager::

        with Client() as client:
            enhet = client.get_enhet("915501680")

    Or by manually opening and closing the client::

        client = Client()
        enhet = client.get_enhet("915501680")
        client.close()
    """

    _client: httpx.Client

    def __init__(self) -> None:
        self.open()

    def __enter__(self) -> Client:
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None = None,
        exc_value: BaseException | None = None,
        traceback: TracebackType | None = None,
    ) -> None:
        self.close()

    def open(self) -> None:
        """Prepare the client for use.

        This is called automatically when the client is created.
        """
        self._client = httpx.Client(
            base_url="https://data.brreg.no/enhetsregisteret/api",
        )

    def close(self) -> None:
        """Close the client and any open HTTP connections.

        This is called automatically if the client is used as a context manager.
        """
        self._client.close()

    def get_enhet(
        self,
        organisasjonsnummer: Organisasjonsnummer,
    ) -> Optional[Enhet]:
        """Get :class:`Enhet` given an organization number."""
        OrganisasjonsnummerValidator.validate_python(organisasjonsnummer)
        with error_handler():
            res = self._client.get(
                f"/enheter/{organisasjonsnummer}",
                headers={
                    "accept": (
                        "application/vnd.brreg.enhetsregisteret.enhet.v2+json;"
                        "charset=UTF-8"
                    )
                },
            )
            if res.status_code in (404, 410):
                return None
            res.raise_for_status()
            return Enhet.model_validate_json(res.content)

    def get_underenhet(
        self,
        organisasjonsnummer: Organisasjonsnummer,
    ) -> Optional[Underenhet]:
        """Get :class:`Underenhet` given an organization number."""
        OrganisasjonsnummerValidator.validate_python(organisasjonsnummer)
        with error_handler():
            res = self._client.get(
                f"/underenheter/{organisasjonsnummer}",
                headers={
                    "accept": (
                        "application/vnd.brreg.enhetsregisteret.underenhet.v2+json;"
                        "charset=UTF-8"
                    )
                },
            )
            if res.status_code in (404, 410):
                return None
            res.raise_for_status()
            return Underenhet.model_validate_json(res.content)

    def search_enhet(
        self,
        query: EnhetQuery,
    ) -> EnhetPage:
        """Search for :class:`Enhet` that matches the given query.

        :param query: The search query.
        """
        with error_handler():
            res = self._client.get(
                f"/enheter?{query.as_url_query()}",
                headers={
                    "accept": (
                        "application/vnd.brreg.enhetsregisteret.enhet.v2+json;"
                        "charset=UTF-8"
                    )
                },
            )
            res.raise_for_status()
            return EnhetPage.model_validate_json(res.content)

    def search_underenhet(
        self,
        query: UnderenhetQuery,
    ) -> UnderenhetPage:
        """Search for :class:`Underenhet` that matches the given query.

        :param query: The search query.
        """
        with error_handler():
            res = self._client.get(
                f"/underenheter?{query.as_url_query()}",
                headers={
                    "accept": (
                        "application/vnd.brreg.enhetsregisteret.underenhet.v2+json;"
                        "charset=UTF-8"
                    )
                },
            )
            res.raise_for_status()
            return UnderenhetPage.model_validate_json(res.content)


@contextmanager
def error_handler() -> Generator[None, Any, None]:
    try:
        yield
    except httpx.HTTPError as exc:
        response: httpx.Response | None = getattr(exc, "response", None)
        raise BrregRestError(
            str(exc),
            method=(exc.request.method if exc.request else None),
            url=(str(exc.request.url) if exc.request else None),
            status_code=(response.status_code if response else None),
        ) from exc
    except Exception as exc:
        raise BrregError(exc) from exc



from typing import Generic, List, TypeVar

from pydantic import (
    AliasPath,
    BaseModel,
    Field,
)

from brreg.enhetsregisteret._responses import Enhet, Underenhet

__all__ = [
    "EnhetPage",
    "UnderenhetPage",
]


T = TypeVar("T", bound=BaseModel)


class Page(BaseModel, Generic[T]):
    """The fields here are available on all page objects."""

    #: The items on this page.
    items: List[T]

    #: The number of elements on this page.
    page_size: int = Field(
        validation_alias=AliasPath("page", "size"),
    )

    #: The page number, starting at 0.
    page_number: int = Field(
        validation_alias=AliasPath("page", "number"),
    )

    #: The total number of elements available.
    total_elements: int = Field(
        validation_alias=AliasPath("page", "totalElements"),
    )

    #: The total number of pages available.
    total_pages: int = Field(
        validation_alias=AliasPath("page", "totalPages"),
    )


class EnhetPage(Page[Enhet]):
    """Response type for enhet search."""

    items: List[Enhet] = Field(
        validation_alias=AliasPath("_embedded", "enheter"),
    )


class UnderenhetPage(Page[Underenhet]):
    """Response type for underenhet search."""

    items: List[Underenhet] = Field(
        validation_alias=AliasPath("_embedded", "underenheter"),
    )



from typing import Optional


class BrregError(Exception):
    """Top-level exception.

    All exceptions raised by the :mod:`brreg` library are subclasses of this
    exception.
    """


class BrregRestError(BrregError):
    """REST API exception."""

    def __init__(
        self,
        msg: str,
        *,
        method: Optional[str],
        url: Optional[str],
        status_code: Optional[int],
    ) -> None:
        super().__init__(f"REST API exception: {msg}")
        self.method = method
        self.url = url
        self.status_code = status_code



from typing import List, Optional

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
)
from pydantic.alias_generators import to_camel

from brreg.enhetsregisteret._types import DateOrNone

__all__ = [
    "Adresse",
    "Enhet",
    "InstitusjonellSektor",
    "Naering",
    "Organisasjonsform",
]


class InstitusjonellSektor(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)

    #: Sektorkoden
    kode: Optional[str] = None

    #: Tekstlig beskrivelse av sektorkoden
    beskrivelse: Optional[str] = None


class Adresse(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)

    #: Adresse
    adresse: List[Optional[str]] = Field(default_factory=list)

    #: Postnummer
    postnummer: Optional[str] = None

    #: Poststed
    poststed: Optional[str] = None

    #: Kommunenummer
    kommunenummer: Optional[str] = None

    #: Kommune
    kommune: Optional[str] = None

    #: Landkode
    landkode: Optional[str] = None

    #: Land
    land: Optional[str] = None


class Naering(BaseModel):
    """Næringskode.

    Næringskoden skal vise virksomhetens hovedaktivitet, og den skal primært
    dekke statistiske behov for Statistisk sentralbyrå (SSB).
    """

    model_config = ConfigDict(alias_generator=to_camel)

    #: Næringskoden
    kode: Optional[str] = None

    #: Tekstlig beskrivelse av næringskoden
    beskrivelse: Optional[str] = None


class Organisasjonsform(BaseModel):
    """Organisasjonsform er virksomhetens formelle organisering.

    Organisasjonsform gir retningslinjer overfor blant annet ansvarsforhold,
    skatt, revisjonsplikt, rettigheter og plikter.
    """

    model_config = ConfigDict(alias_generator=to_camel)

    #: Organisasjonsformen
    kode: str

    #: Tekstlig beskrivelse av organisasjonsformen
    beskrivelse: str

    #: Dato når organisasjonsformen evt. ble ugyldig
    utgaatt: DateOrNone = None


class Enhet(BaseModel):
    """Enhet på øverste nivå i registreringsstrukturen i Enhetsregisteret.

    Eksempelvis enkeltpersonforetak, foreninger, selskap, sameier og andre som
    er registrert i Enhetsregisteret. Identifiseres med organisasjonsnummer.
    """

    model_config = ConfigDict(alias_generator=to_camel)

    #: Organisasjonsnummer
    organisasjonsnummer: str

    #: Navn
    navn: str

    #: Organisasjonsform
    organisasjonsform: Organisasjonsform

    #: Hjemmeside
    hjemmeside: Optional[str] = None

    #: Enhetens postadresse
    postadresse: Optional[Adresse] = None

    #: Registreringsdato i Enhetsregisteret
    registreringsdato_enhetsregisteret: DateOrNone = None

    #: Hvorvidt enheten er registrert i MVA-registeret
    registrert_i_mvaregisteret: Optional[bool] = None

    #: Enheter som i utgangspunktet ikke er mva-pliktig, kan søke om frivillig
    #: registrering i Merverdiavgiftsregisteret
    frivillig_mva_registrert_beskrivelser: List[str] = Field(default_factory=list)

    #: Næringskode 1
    naeringskode1: Optional[Naering] = None

    #: Næringskode 2
    naeringskode2: Optional[Naering] = None

    #: Næringskode 3
    naeringskode3: Optional[Naering] = None

    #: Hjelpeenhetskode
    hjelpeenhetskode: Optional[Naering] = None

    #: Antall ansatte
    antall_ansatte: Optional[int] = None

    #: Angir om enheten har registrert ansatte
    har_registrert_antall_ansatte: Optional[bool] = None

    #: Organisasjonsnummeret til overordnet enhet i offentlig sektor
    overordnet_enhet: Optional[str] = None

    #: Forretningsadresse
    forretningsadresse: Optional[Adresse] = None

    #: Stiftelsesdato
    stiftelsesdato: DateOrNone = None

    #: Sektorkode
    institusjonell_sektorkode: Optional[InstitusjonellSektor] = None

    #: Hvorvidt enheten er registrert i Foretaksregisteret
    registrert_i_foretaksregisteret: Optional[bool] = None

    #: Hvorvidt enheten er registrert i Stiftelsesregisteret
    registrert_i_stiftelsesregisteret: Optional[bool] = None

    #: Hvorvidt enheten er registrert i Frivillighetsregisteret
    registrert_i_frivillighetsregisteret: Optional[bool] = None

    #: År for siste innsendte årsregnskap
    siste_innsendte_aarsregnskap: Optional[int] = None

    #: Hvorvidt enheten er konkurs
    konkurs: Optional[bool] = None

    #: Kjennelsesdato for konkursen
    konkursdato: DateOrNone = None

    #: Hvorvidt enheten er under avvikling
    under_avvikling: Optional[bool] = None

    #: Hvorvidt enheten er under tvangsavvikling eller tvangsoppløsning
    under_tvangsavvikling_eller_tvangsopplosning: Optional[bool] = None

    #: Målform
    maalform: Optional[str] = None

    #: Enhetens vedtektsdato
    vedtektsdato: DateOrNone = None

    #: Enhetens formål
    vedtektsfestet_formaal: List[str] = Field(default_factory=list)

    #: Enhetens aktivitet
    aktivitet: List[str] = Field(default_factory=list)

    #: Nedleggelsesdato for underenheten
    nedleggelsesdato: DateOrNone = None

    #: Dato under-/enheten ble slettet
    slettedato: DateOrNone = None


class Underenhet(BaseModel):
    """Enhet på laveste nivå i registreringsstrukturen i Enhetsregisteret.

    En underenhet kan ikke eksistere alene og har alltid knytning til en
    hovedenhet. Identifiseres med organisasjonsnummer.
    """

    model_config = ConfigDict(alias_generator=to_camel)

    #: Underenhetens organisasjonsnummer
    organisasjonsnummer: str

    #: Underenhetens navn
    navn: str

    #: Underenhetens navn
    organisasjonsform: Organisasjonsform

    #: Underenhetens hjemmeside
    hjemmeside: Optional[str] = None

    #: Underenhetens postadresse
    postadresse: Optional[Adresse] = None

    #: Underenhetens registreringsdato i Enhetsregisteret
    registreringsdato_enhetsregisteret: DateOrNone = None

    #: Hvorvidt underenheten er registrert i MVA-registeret
    registrert_i_mvaregisteret: Optional[bool] = None

    #: Underenheter som i utgangspunktet ikke er mva-pliktig, kan søke om
    #: frivillig registrering i Merverdiavgiftsregisteret
    frivillig_mva_registrert_beskrivelser: List[str] = Field(default_factory=list)

    #: Næringskode 1
    naeringskode1: Optional[Naering] = None

    #: Næringskode 2
    naeringskode2: Optional[Naering] = None

    #: Næringskode 3
    naeringskode3: Optional[Naering] = None

    #: Hjelpeenhetskode
    hjelpeenhetskode: Optional[Naering] = None

    #: Antall ansatte
    antall_ansatte: Optional[int] = None

    #: Angir om enheten har registrert ansatte
    har_registrert_antall_ansatte: Optional[bool] = None

    #: Underenhetens overordnede enhet
    overordnet_enhet: Optional[str] = None

    #: Underenhetens beliggenhetsadresse
    beliggenhetsadresse: Optional[Adresse] = None

    #: Underenhetens oppstartsdato
    oppstartsdato: DateOrNone = None

    #: Underenhetens dato for eierskifte
    dato_eierskifte: DateOrNone = None

    #: Nedleggelsesdato for underenheten
    nedleggelsesdato: DateOrNone = None

    #: Dato under-/enheten ble slettet
    slettedato: DateOrNone = None



"""API client for `Enhetsregisteret <https://w2.brreg.no/enhet/sok/>`_.

See https://data.brreg.no/enhetsregisteret/api/docs/index.html for API details.
"""

from brreg.enhetsregisteret._client import Client
from brreg.enhetsregisteret._pagination import EnhetPage, Page, UnderenhetPage
from brreg.enhetsregisteret._queries import EnhetQuery, Query, UnderenhetQuery
from brreg.enhetsregisteret._responses import (
    Adresse,
    Enhet,
    InstitusjonellSektor,
    Naering,
    Organisasjonsform,
    Underenhet,
)
from brreg.enhetsregisteret._types import (
    Kommunenummer,
    KommunenummerValidator,
    Organisasjonsnummer,
    OrganisasjonsnummerValidator,
    Postnummer,
    PostnummerValidator,
    Sektorkode,
    SektorkodeValidator,
)

__all__ = [
    # From _client module:
    "Client",
    # From _pagination module:
    "EnhetPage",
    "Page",
    "UnderenhetPage",
    # From _queries module:
    "EnhetQuery",
    "Query",
    "UnderenhetQuery",
    # From _responses module:
    "Adresse",
    "Enhet",
    "InstitusjonellSektor",
    "Naering",
    "Organisasjonsform",
    "Underenhet",
    # From _types module:
    "Kommunenummer",
    "KommunenummerValidator",
    "Organisasjonsnummer",
    "OrganisasjonsnummerValidator",
    "Postnummer",
    "PostnummerValidator",
    "Sektorkode",
    "SektorkodeValidator",
]



import datetime as dt
from typing import List, Optional, TypeVar

from pydantic import (
    BeforeValidator,
    Field,
    PlainSerializer,
    TypeAdapter,
)
from typing_extensions import Annotated

T = TypeVar("T")

# A list that serializes to a comma-separated string.
CommaList = Annotated[
    List[T],
    PlainSerializer(
        lambda v: ",".join(v),
        return_type=str,
        when_used="json",
    ),
]

# Same as `Optional[dt.date]`, except that this version deserializes empty
# strings to `None`.
DateOrNone = Annotated[
    Optional[dt.date], BeforeValidator(lambda v: v if v != "" else None)
]

Kommunenummer = Annotated[
    str,
    Field(min_length=4, max_length=4, pattern=r"^\d{4}$"),
]
KommunenummerValidator = TypeAdapter(Kommunenummer)

Naeringskode = Annotated[
    str,
    Field(min_length=6, max_length=6, pattern=r"^\d{2}\.\d{3}$"),
]
NaeringskodeValidator = TypeAdapter(Naeringskode)

Organisasjonsnummer = Annotated[
    str,
    BeforeValidator(lambda v: v.replace(" ", "")),
    Field(min_length=9, max_length=9, pattern=r"^\d{9}$"),
]
OrganisasjonsnummerValidator = TypeAdapter(Organisasjonsnummer)

Postnummer = Annotated[
    str,
    Field(min_length=4, max_length=4, pattern=r"^\d{4}$"),
]
PostnummerValidator = TypeAdapter(Postnummer)

Sektorkode = Annotated[
    str,
    Field(min_length=4, max_length=4, pattern=r"^\d{4}$"),
]
SektorkodeValidator = TypeAdapter(Sektorkode)
